@@ -15,6 +15,7 @@
 import contextlib
 import errno
 import hashlib
+import importlib
 import logging
 import math
 import os
@@ -32,7 +33,7 @@ from glance_store import capabilities
 from glance_store.common import utils
 import glance_store.driver
 from glance_store import exceptions
-from glance_store.i18n import _, _LE, _LI
+from glance_store.i18n import _, _LE, _LI, _LW
 import glance_store.location
 
 try:
@@ -52,7 +53,7 @@ LOG = logging.getLogger(__name__)
 
 _CINDER_OPTS = [
     cfg.StrOpt('cinder_catalog_info',
-               default='volumev2::publicURL',
+               default='volumev3::publicURL',
                help="""
 Information to match when looking for cinder in the service catalog.
 
@@ -304,6 +305,11 @@ Possible values:
 Related options:
     * None
 
+NOTE: You cannot use an encrypted volume_type associated with an NFS backend.
+An encrypted volume stored on an NFS backend will raise an exception whenever
+glance_store tries to write or access image data stored in that volume.
+Consult your Cinder administrator to determine an appropriate volume_type.
+
 """),
     cfg.BoolOpt('cinder_enforce_multipath',
                 default=False,
@@ -354,13 +360,16 @@ class StoreLocation(glance_store.location.StoreLocation):
         self.volume_id = self.specs.get('volume_id')
 
     def get_uri(self):
+        if self.backend_group:
+            return "cinder://%s/%s" % (self.backend_group,
+                                       self.volume_id)
         return "cinder://%s" % self.volume_id
 
     def parse_uri(self, uri):
         self.validate_schemas(uri, valid_schemas=('cinder://',))
 
         self.scheme = 'cinder'
-        self.volume_id = uri[9:]
+        self.volume_id = uri.split('/')[-1]
 
         if not utils.is_uuid_like(self.volume_id):
             reason = _("URI contains invalid volume ID")
@@ -380,54 +389,114 @@ class Store(glance_store.driver.Store):
 
     def __init__(self, *args, **kargs):
         super(Store, self).__init__(*args, **kargs)
+        # We are importing it here to let the config options load
+        # before we use them in the fs_mount file
+        self.mount = importlib.import_module('glance_store.common.fs_mount')
+        self._set_url_prefix()
         if self.backend_group:
-            self._set_url_prefix()
+            self.store_conf = getattr(self.conf, self.backend_group)
+        else:
+            self.store_conf = self.conf.glance_store
+
+    def _set_url_prefix(self):
+        self._url_prefix = "cinder://"
+        if self.backend_group:
+            self._url_prefix = "cinder://%s" % self.backend_group
+
+    def configure_add(self):
+        """
+        Check to verify if the volume types configured for the cinder store
+        exist in deployment and if not, log a warning.
+        """
+        cinder_volume_type = self.store_conf.cinder_volume_type
+        if cinder_volume_type:
+            # NOTE: `cinder_volume_type` is configured, check
+            # configured volume_type is available in cinder or not
+            cinder_client = self.get_cinderclient()
+            try:
+                # We don't even need the volume type object, as long
+                # as this returns clean, we know the name is good.
+                cinder_client.volume_types.find(name=cinder_volume_type)
+                # No need to worry about a NoUniqueMatch as volume type name
+                # is unique
+            except cinder_exception.NotFound:
+                reason = (_LW("Invalid `cinder_volume_type %s`"
+                              % cinder_volume_type))
+                LOG.warning(reason)
+            except cinder_exception.ClientException:
+                pass
+
+    def is_image_associated_with_store(self, context, volume_id):
+        """
+        Updates legacy images URL to respective stores.
+        This method checks the volume type of the volume associated with the
+        image against the configured stores. It returns true if the
+        cinder_volume_type configured in the store matches with the volume
+        type of the image-volume. When cinder_volume_type is not configured
+        then the it checks it against default_volume_type set in cinder.
+        If above both conditions doesn't meet, it returns false.
+        """
+        try:
+            cinder_client = self.get_cinderclient(context=context,
+                                                  legacy_update=True)
+            cinder_volume_type = self.store_conf.cinder_volume_type
+            volume = cinder_client.volumes.get(volume_id)
+            if cinder_volume_type and volume.volume_type == cinder_volume_type:
+                return True
+            elif not cinder_volume_type:
+                default_type = cinder_client.volume_types.default()
+                if volume.volume_type == default_type.name:
+                    return True
+        except Exception:
+            # Glance calls this method to update legacy images URL
+            # If an exception occours due to image/volume is non-existent or
+            # any other reason, we return False (i.e. the image location URL
+            # won't be updated) and it is glance's responsibility to handle
+            # the case when the image failed to update
+            pass
+
+        return False
 
     def get_root_helper(self):
-        if self.backend_group:
-            rootwrap = getattr(CONF, self.backend_group).rootwrap_config
-        else:
-            rootwrap = CONF.glance_store.rootwrap_config
-
+        rootwrap = self.store_conf.rootwrap_config
         return 'sudo glance-rootwrap %s' % rootwrap
 
     def is_user_overriden(self):
-        if self.backend_group:
-            store_conf = getattr(self.conf, self.backend_group)
-        else:
-            store_conf = self.conf.glance_store
-
-        return all([store_conf.get('cinder_store_' + key)
+        return all([self.store_conf.get('cinder_store_' + key)
                     for key in ['user_name', 'password',
                                 'project_name', 'auth_address']])
 
-    def get_cinderclient(self, context=None):
-        if self.backend_group:
-            glance_store = getattr(self.conf, self.backend_group)
+    def get_cinderclient(self, context=None, legacy_update=False):
+        # NOTE: For legacy image update from single store to multiple
+        # stores we need to use admin context rather than user provided
+        # credentials
+        if legacy_update:
+            user_overriden = False
+            context = context.elevated()
         else:
-            glance_store = self.conf.glance_store
+            user_overriden = self.is_user_overriden()
 
-        user_overriden = self.is_user_overriden()
         if user_overriden:
-            username = glance_store.cinder_store_user_name
-            password = glance_store.cinder_store_password
-            project = glance_store.cinder_store_project_name
-            url = glance_store.cinder_store_auth_address
+            username = self.store_conf.cinder_store_user_name
+            password = self.store_conf.cinder_store_password
+            project = self.store_conf.cinder_store_project_name
+            url = self.store_conf.cinder_store_auth_address
         else:
-            username = context.user
+            username = context.user_id
             password = context.auth_token
-            project = context.tenant
+            project = context.project_id
 
-            if glance_store.cinder_endpoint_template:
-                url = glance_store.cinder_endpoint_template % context.to_dict()
+            if self.store_conf.cinder_endpoint_template:
+                template = self.store_conf.cinder_endpoint_template
+                url = template % context.to_dict()
             else:
-                info = glance_store.cinder_catalog_info
+                info = self.store_conf.cinder_catalog_info
                 service_type, service_name, interface = info.split(':')
                 try:
                     catalog = keystone_sc.ServiceCatalogV2(
                         context.service_catalog)
                     url = catalog.url_for(
-                        region_name=glance_store.cinder_os_region_name,
+                        region_name=self.store_conf.cinder_os_region_name,
                         service_type=service_type,
                         service_name=service_name,
                         interface=interface)
@@ -438,10 +507,10 @@ class Store(glance_store.driver.Store):
 
         c = cinderclient.Client(
             username, password, project, auth_url=url,
-            region_name=glance_store.cinder_os_region_name,
-            insecure=glance_store.cinder_api_insecure,
-            retries=glance_store.cinder_http_retries,
-            cacert=glance_store.cinder_ca_certificates_file)
+            region_name=self.store_conf.cinder_os_region_name,
+            insecure=self.store_conf.cinder_api_insecure,
+            retries=self.store_conf.cinder_http_retries,
+            cacert=self.store_conf.cinder_ca_certificates_file)
 
         LOG.debug(
             'Cinderclient connection created for user %(user)s using URL: '
@@ -476,9 +545,6 @@ class Store(glance_store.driver.Store):
     def get_schemes(self):
         return ('cinder',)
 
-    def _set_url_prefix(self):
-        self._url_prefix = "cinder://"
-
     def _check_context(self, context, require_tenant=False):
         user_overriden = self.is_user_overriden()
         if user_overriden and not require_tenant:
@@ -494,12 +560,7 @@ class Store(glance_store.driver.Store):
 
     def _wait_volume_status(self, volume, status_transition, status_expected):
         max_recheck_wait = 15
-        if self.backend_group:
-            timeout = getattr(
-                self.conf, self.backend_group).cinder_state_transition_timeout
-        else:
-            timeout = self.conf.glance_store.cinder_state_transition_timeout
-
+        timeout = self.store_conf.cinder_state_transition_timeout
         volume = volume.manager.get(volume.id)
         tries = 0
         elapsed = 0
@@ -525,6 +586,22 @@ class Store(glance_store.driver.Store):
             raise exceptions.BackendException(msg)
         return volume
 
+    def get_hash_str(self, base_str):
+        """Returns string that represents SHA256 hash of base_str (in hex format).
+
+        If base_str is a Unicode string, encode it to UTF-8.
+        """
+        if isinstance(base_str, str):
+            base_str = base_str.encode('utf-8')
+        return hashlib.sha256(base_str).hexdigest()
+
+    def _get_mount_path(self, share, mount_point_base):
+        """Returns the mount path prefix using the mount point base and share.
+
+        :returns: The mount path prefix.
+        """
+        return os.path.join(mount_point_base, self.get_hash_str(share))
+
     @contextlib.contextmanager
     def _open_cinder_volume(self, client, volume, mode):
         attach_mode = 'rw' if mode == 'wb' else 'ro'
@@ -532,17 +609,9 @@ class Store(glance_store.driver.Store):
         root_helper = self.get_root_helper()
         priv_context.init(root_helper=shlex.split(root_helper))
         host = socket.gethostname()
-        if self.backend_group:
-            use_multipath = getattr(
-                self.conf, self.backend_group).cinder_use_multipath
-            enforce_multipath = getattr(
-                self.conf, self.backend_group).cinder_enforce_multipath
-            mount_point_base = getattr(
-                self.conf, self.backend_group).cinder_mount_point_base
-        else:
-            use_multipath = self.conf.glance_store.cinder_use_multipath
-            enforce_multipath = self.conf.glance_store.cinder_enforce_multipath
-            mount_point_base = self.conf.glance_store.cinder_mount_point_base
+        use_multipath = self.store_conf.cinder_use_multipath
+        enforce_multipath = self.store_conf.cinder_enforce_multipath
+        mount_point_base = self.store_conf.cinder_mount_point_base
 
         properties = connector.get_connector_properties(
             root_helper, host, use_multipath, enforce_multipath)
@@ -557,13 +626,36 @@ class Store(glance_store.driver.Store):
 
         try:
             connection_info = volume.initialize_connection(volume, properties)
-            if connection_info['driver_volume_type'] == 'nfs':
-                connection_info['mount_point_base'] = os.path.join(
-                    mount_point_base, 'nfs')
             conn = connector.InitiatorConnector.factory(
                 connection_info['driver_volume_type'], root_helper,
                 conn=connection_info)
-            device = conn.connect_volume(connection_info['data'])
+            if connection_info['driver_volume_type'] == 'nfs':
+                if volume.encrypted:
+                    volume.unreserve(volume)
+                    volume.delete()
+                    msg = (_('Encrypted volume creation for cinder nfs is not '
+                             'supported from glance_store. Failed to create '
+                             'volume %(volume_id)s')
+                           % {'volume_id': volume.id})
+                    LOG.error(msg)
+                    raise exceptions.BackendException(msg)
+
+                @utils.synchronized(connection_info['data']['export'])
+                def connect_volume_nfs():
+                    data = connection_info['data']
+                    export = data['export']
+                    vol_name = data['name']
+                    mountpoint = self._get_mount_path(
+                        export,
+                        os.path.join(mount_point_base, 'nfs'))
+                    options = data['options']
+                    self.mount.mount(
+                        'nfs', export, vol_name, mountpoint, host,
+                        root_helper, options)
+                    return {'path': os.path.join(mountpoint, vol_name)}
+                device = connect_volume_nfs()
+            else:
+                device = conn.connect_volume(connection_info['data'])
             volume.attach(None, 'glance_store', attach_mode, host_name=host)
             volume = self._wait_volume_status(volume, 'attaching', 'in-use')
             if (connection_info['driver_volume_type'] == 'rbd' and
@@ -571,8 +663,7 @@ class Store(glance_store.driver.Store):
                 yield device['path']
             else:
                 with self.temporary_chown(
-                        device['path'], backend=self.backend_group
-                ), open(device['path'], mode) as f:
+                        device['path']), open(device['path'], mode) as f:
                     yield f
         except Exception:
             LOG.exception(_LE('Exception while accessing to cinder volume '
@@ -586,7 +677,15 @@ class Store(glance_store.driver.Store):
 
             if device:
                 try:
-                    conn.disconnect_volume(connection_info['data'], device)
+                    if connection_info['driver_volume_type'] == 'nfs':
+                        @utils.synchronized(connection_info['data']['export'])
+                        def disconnect_volume_nfs():
+                            path, vol_name = device['path'].rsplit('/', 1)
+                            self.mount.umount(vol_name, path, host,
+                                              root_helper)
+                        disconnect_volume_nfs()
+                    else:
+                        conn.disconnect_volume(connection_info['data'], device)
                 except Exception:
                     LOG.exception(_LE('Failed to disconnect volume '
                                       '%(volume_id)s.'),
@@ -718,23 +817,19 @@ class Store(glance_store.driver.Store):
 
         self._check_context(context, require_tenant=True)
         client = self.get_cinderclient(context)
-        os_hash_value = hashlib.new(str(hashing_algo))
-        checksum = hashlib.md5()
+        os_hash_value = utils.get_hasher(hashing_algo, False)
+        checksum = utils.get_hasher('md5', False)
         bytes_written = 0
         size_gb = int(math.ceil(float(image_size) / units.Gi))
         if size_gb == 0:
             size_gb = 1
         name = "image-%s" % image_id
-        owner = context.tenant
+        owner = context.project_id
         metadata = {'glance_image_id': image_id,
                     'image_size': str(image_size),
                     'image_owner': owner}
 
-        if self.backend_group:
-            volume_type = getattr(self.conf,
-                                  self.backend_group).cinder_volume_type
-        else:
-            volume_type = self.conf.glance_store.cinder_volume_type
+        volume_type = self.store_conf.cinder_volume_type
 
         LOG.debug('Creating a new volume: image_size=%d size_gb=%d type=%s',
                   image_size, size_gb, volume_type or 'None')
@@ -742,8 +837,18 @@ class Store(glance_store.driver.Store):
             LOG.info(_LI("Since image size is zero, we will be doing "
                          "resize-before-write for each GB which "
                          "will be considerably slower than normal."))
-        volume = client.volumes.create(size_gb, name=name, metadata=metadata,
-                                       volume_type=volume_type)
+        try:
+            volume = client.volumes.create(size_gb, name=name,
+                                           metadata=metadata,
+                                           volume_type=volume_type)
+        except cinder_exception.NotFound:
+            LOG.error(_LE("Invalid volume type %s configured. Please check "
+                          "the `cinder_volume_type` configuration parameter."
+                          % volume_type))
+            msg = (_("Failed to create image-volume due to invalid "
+                     "`cinder_volume_type` configured."))
+            raise exceptions.BackendException(msg)
+
         volume = self._wait_volume_status(volume, 'creating', 'available')
         size_gb = volume.size
 
@@ -818,10 +923,13 @@ class Store(glance_store.driver.Store):
                    'checksum_hex': checksum_hex})
 
         image_metadata = {}
+        location_url = 'cinder://%s' % volume.id
         if self.backend_group:
             image_metadata['store'] = u"%s" % self.backend_group
+            location_url = 'cinder://%s/%s' % (self.backend_group,
+                                               volume.id)
 
-        return ('cinder://%s' % volume.id,
+        return (location_url,
                 bytes_written,
                 checksum_hex,
                 hash_hex,
