@@ -1,4 +1,5 @@
 # Copyright 2010-2011 Josh Durgin
+# Copyright 2020 Red Hat, Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,11 +16,8 @@
 
 """Storage backend for RBD
    (RADOS (Reliable Autonomic Distributed Object Store) Block Device)"""
-from __future__ import absolute_import
-from __future__ import with_statement
 
 import contextlib
-import hashlib
 import logging
 import math
 
@@ -32,7 +30,7 @@ from glance_store import capabilities
 from glance_store.common import utils
 from glance_store import driver
 from glance_store import exceptions
-from glance_store.i18n import _, _LE, _LI
+from glance_store.i18n import _, _LE, _LI, _LW
 from glance_store import location
 
 try:
@@ -151,6 +149,26 @@ Possible Values:
 Related options:
     * None
 
+"""),
+    cfg.BoolOpt('rbd_thin_provisioning',
+                default=False,
+                help="""
+Enable or not thin provisioning in this backend.
+
+This configuration option enable the feature of not really write null byte
+sequences on the RBD backend, the holes who can appear will automatically
+be interpreted by Ceph as null bytes, and do not really consume your storage.
+Enabling this feature will also speed up image upload and save network trafic
+in addition to save space in the backend, as null bytes sequences are not
+sent over the network.
+
+Possible Values:
+    * True
+    * False
+
+Related options:
+    * None
+
 """),
 ]
 
@@ -271,10 +289,16 @@ class Store(driver.Store):
 
         try:
             client.connect(timeout=self.connect_timeout)
-        except rados.Error:
-            msg = _LE("Error connecting to ceph cluster.")
-            LOG.exception(msg)
-            raise exceptions.BackendException()
+        except (rados.Error, rados.ObjectNotFound) as e:
+            if self.backend_group and len(self.conf.enabled_backends) > 1:
+                reason = _("Error in store configuration: %s") % e
+                LOG.debug(reason)
+                raise exceptions.BadStoreConfiguration(
+                    store_name=self.backend_group, reason=reason)
+            else:
+                msg = _LE("Error connecting to ceph cluster.")
+                LOG.exception(msg)
+                raise exceptions.BackendException()
         try:
             yield client
         finally:
@@ -297,13 +321,19 @@ class Store(driver.Store):
                                     self.backend_group).rbd_store_ceph_conf
                 connect_timeout = getattr(
                     self.conf, self.backend_group).rados_connect_timeout
+                thin_provisioning = getattr(self.conf,
+                                            self.backend_group).\
+                    rbd_thin_provisioning
             else:
                 chunk = self.conf.glance_store.rbd_store_chunk_size
                 pool = self.conf.glance_store.rbd_store_pool
                 user = self.conf.glance_store.rbd_store_user
                 conf_file = self.conf.glance_store.rbd_store_ceph_conf
                 connect_timeout = self.conf.glance_store.rados_connect_timeout
+                thin_provisioning = \
+                    self.conf.glance_store.rbd_thin_provisioning
 
+            self.thin_provisioning = thin_provisioning
             self.chunk_size = chunk * units.Mi
             self.READ_CHUNKSIZE = self.chunk_size
             self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
@@ -321,6 +351,8 @@ class Store(driver.Store):
                                                    reason=reason)
         if self.backend_group:
             self._set_url_prefix()
+        self.size = 0
+        self.resize_amount = self.WRITE_CHUNKSIZE
 
     def _set_url_prefix(self):
         fsid = None
@@ -430,26 +462,26 @@ class Store(driver.Store):
                                        {'snap_exc': exc})
                                 LOG.debug(msg)
                             except rbd.ImageBusy as exc:
-                                log_msg = (_LE("Snap Operating Exception "
+                                log_msg = (_LW("Snap Operating Exception "
                                                "%(snap_exc)s "
                                                "Snapshot is in use.") %
                                            {'snap_exc': exc})
-                                LOG.error(log_msg)
+                                LOG.warning(log_msg)
                                 raise exceptions.InUseByStore()
 
                     # Then delete image.
                     rbd.RBD().remove(ioctx, image_name)
                 except rbd.ImageHasSnapshots:
-                    log_msg = (_LE("Remove image %(img_name)s failed. "
+                    log_msg = (_LW("Remove image %(img_name)s failed. "
                                    "It has snapshot(s) left.") %
                                {'img_name': image_name})
-                    LOG.error(log_msg)
+                    LOG.warning(log_msg)
                     raise exceptions.HasSnapshot()
                 except rbd.ImageBusy:
-                    log_msg = (_LE("Remove image %(img_name)s failed. "
+                    log_msg = (_LW("Remove image %(img_name)s failed. "
                                    "It is in use.") %
                                {'img_name': image_name})
-                    LOG.error(log_msg)
+                    LOG.warning(log_msg)
                     raise exceptions.InUseByStore()
                 except rbd.ImageNotFound:
                     msg = _("RBD image %s does not exist") % image_name
@@ -464,6 +496,18 @@ class Store(driver.Store):
             # Such exception is not dangerous for us so it will be just logged
             LOG.debug("Snapshot %s is unprotected already" % snap_name)
 
+    def _resize_on_write(self, image, image_size, bytes_written, chunk_length):
+        """Handle the rbd resize when needed."""
+        if image_size != 0 or self.size >= bytes_written + chunk_length:
+            return self.size
+        new_size = self.size + self.resize_amount
+        LOG.debug("resizing image to %s KiB" % (new_size / units.Ki))
+        image.resize(new_size)
+        # Note(jokke): We double how much we grow the image each time
+        # up to 8gigs to avoid resizing for each write on bigger images
+        self.resize_amount = min(self.resize_amount * 2, 8 * units.Gi)
+        return new_size
+
     @driver.back_compat_add
     @capabilities.check
     def add(self, image_id, image_file, image_size, hashing_algo, context=None,
@@ -486,8 +530,8 @@ class Store(driver.Store):
         :raises: `glance_store.exceptions.Duplicate` if the image already
                  exists
         """
-        checksum = hashlib.md5()
-        os_hash_value = hashlib.new(str(hashing_algo))
+        os_hash_value = utils.get_hasher(hashing_algo, False)
+        checksum = utils.get_hasher('md5', False)
         image_name = str(image_id)
         with self.get_connection(conffile=self.conf_file,
                                  rados_id=self.user) as conn:
@@ -510,9 +554,9 @@ class Store(driver.Store):
                 LOG.debug('creating image %s with order %d and size %d',
                           image_name, order, image_size)
                 if image_size == 0:
-                    LOG.warning(_("since image size is zero we will be doing "
-                                  "resize-before-write for each chunk which "
-                                  "will be considerably slower than normal"))
+                    LOG.warning(_LW("Since image size is zero we will be "
+                                    "doing resize-before-write which will be "
+                                    "slower than normal"))
 
                 try:
                     loc = self._create_image(fsid, conn, ioctx, image_name,
@@ -528,24 +572,27 @@ class Store(driver.Store):
                         chunks = utils.chunkreadable(image_file,
                                                      self.WRITE_CHUNKSIZE)
                         for chunk in chunks:
-                            # If the image size provided is zero we need to do
-                            # a resize for the amount we are writing. This will
-                            # be slower so setting a higher chunk size may
-                            # speed things up a bit.
-                            if image_size == 0:
-                                chunk_length = len(chunk)
-                                length = offset + chunk_length
-                                bytes_written += chunk_length
-                                LOG.debug(_("resizing image to %s KiB") %
-                                          (length / units.Ki))
-                                image.resize(length)
-                            LOG.debug(_("writing chunk at offset %s") %
-                                      (offset))
-                            offset += image.write(chunk, offset)
+                            # NOTE(jokke): If we don't know image size we need
+                            # to resize it on write. The resize amount will
+                            # ramp up to 8 gigs.
+                            chunk_length = len(chunk)
+                            self.size = self._resize_on_write(image,
+                                                              image_size,
+                                                              bytes_written,
+                                                              chunk_length)
+                            bytes_written += chunk_length
+                            if not (self.thin_provisioning and not any(chunk)):
+                                image.write(chunk, offset)
+                            offset += chunk_length
                             os_hash_value.update(chunk)
                             checksum.update(chunk)
                             if verifier:
                                 verifier.update(chunk)
+
+                        # Lets trim the image in case we overshoot with resize
+                        if image_size == 0:
+                            image.resize(bytes_written)
+
                         if loc.snapshot:
                             image.create_snap(loc.snapshot)
                             image.protect_snap(loc.snapshot)
