@@ -15,7 +15,6 @@
 
 """Storage backend for SWIFT"""
 
-import hashlib
 import logging
 import math
 
@@ -40,6 +39,7 @@ from glance_store._drivers.swift import buffered
 from glance_store._drivers.swift import connection_manager
 from glance_store._drivers.swift import utils as sutils
 from glance_store import capabilities
+from glance_store.common import utils as gutils
 from glance_store import driver
 from glance_store import exceptions
 from glance_store.i18n import _, _LE, _LI
@@ -768,6 +768,7 @@ def Store(conf, backend=None):
         return MultiTenantStore(conf, backend=backend)
     return SingleTenantStore(conf, backend=backend)
 
+
 Store.OPTIONS = _SWIFT_OPTS + sutils.swift_opts + buffered.BUFFERING_OPTS
 
 
@@ -930,7 +931,7 @@ class BaseStore(driver.Store):
         :raises: `glance_store.exceptions.Duplicate` if something already
                 exists at this location
         """
-        os_hash_value = hashlib.new(str(hashing_algo))
+        os_hash_value = gutils.get_hasher(hashing_algo, False)
         location = self.create_location(image_id, context=context)
         # initialize a manager with re-auth if image need to be splitted
         need_chunks = (image_size == 0) or (
@@ -947,7 +948,7 @@ class BaseStore(driver.Store):
                 if not need_chunks:
                     # Image size is known, and is less than large_object_size.
                     # Send to Swift with regular PUT.
-                    checksum = hashlib.md5()
+                    checksum = gutils.get_hasher('md5', False)
                     reader = ChunkReader(image_file, checksum,
                                          os_hash_value, image_size,
                                          verifier=verifier)
@@ -972,7 +973,7 @@ class BaseStore(driver.Store):
                                   "Swift.")
                         total_chunks = '?'
 
-                    checksum = hashlib.md5()
+                    checksum = gutils.get_hasher('md5', False)
                     written_chunks = []
                     combined_chunks_size = 0
                     while True:
@@ -1122,7 +1123,7 @@ class BaseStore(driver.Store):
                     try:
                         connection.delete_object(obj_container,
                                                  segment['name'])
-                    except swiftclient.ClientException as e:
+                    except swiftclient.ClientException:
                         msg = _('Unable to delete segment %(segment_name)s')
                         msg = msg % {'segment_name': segment['name']}
                         LOG.exception(msg)
@@ -1598,7 +1599,7 @@ class MultiTenantStore(BaseStore):
         # create client for multitenant user(trustor)
         trustor_auth = ks_identity.V3Token(auth_url=auth_address,
                                            token=context.auth_token,
-                                           project_id=context.tenant)
+                                           project_id=context.project_id)
         trustor_sess = ks_session.Session(auth=trustor_auth,
                                           verify=self.ks_verify)
         trustor_client = ks_client.Client(session=trustor_sess)
@@ -1625,8 +1626,8 @@ class MultiTenantStore(BaseStore):
 
         # create trust for trustee user
         trust_id = trustor_client.trusts.create(
-            trustee_user=trustee_user_id, trustor_user=context.user,
-            project=context.tenant, impersonation=True,
+            trustee_user=trustee_user_id, trustor_user=context.user_id,
+            project=context.project_id, impersonation=True,
             role_names=roles
         ).id
         # initialize a new client with trust and trustee credentials
