@@ -15,7 +15,6 @@
 
 """Storage backend for VMware Datastore"""
 
-import hashlib
 import logging
 import os
 
@@ -261,8 +260,8 @@ class _Reader(object):
     def __init__(self, data, hashing_algo, verifier=None):
         self._size = 0
         self.data = data
-        self.checksum = hashlib.md5()
-        self.os_hash_value = hashlib.new(str(hashing_algo))
+        self.os_hash_value = utils.get_hasher(hashing_algo, False)
+        self.checksum = utils.get_hasher('md5', False)
         self.verifier = verifier
 
     def read(self, size=None):
@@ -515,6 +514,19 @@ class Store(glance_store.Store):
             store_conf = self.conf.glance_store
 
         self.store_image_dir = store_conf.vmware_store_image_dir
+        if self.backend_group:
+            self._set_url_prefix()
+
+    def _set_url_prefix(self):
+        path = os.path.join(DS_URL_PREFIX,
+                            self.store_image_dir)
+
+        if netutils.is_valid_ipv6(self.server_host):
+            self._url_prefix = '%s://[%s]%s' % (self.scheme,
+                                                self.server_host, path)
+        else:
+            self._url_prefix = '%s://%s%s' % (self.scheme,
+                                              self.server_host, path)
 
     def select_datastore(self, image_size):
         """Select a datastore with free space larger than image size."""
@@ -644,7 +656,7 @@ class Store(glance_store.Store):
 
         metadata = {}
         if self.backend_group:
-            metadata['backend'] = u"%s" % self.backend_group
+            metadata['store'] = u"%s" % self.backend_group
 
         return (loc.get_uri(),
                 image_file.size,
