@@ -16,10 +16,12 @@
 import contextlib
 import errno
 import hashlib
-import mock
 import os
+from unittest import mock
+
 import six
 import socket
+import sys
 import tempfile
 import time
 import uuid
@@ -27,14 +29,17 @@ import uuid
 from cinderclient.v3 import client as cinderclient
 from os_brick.initiator import connector
 from oslo_concurrency import processutils
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 
-from glance_store._drivers import cinder
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
 from glance_store.tests.unit import test_store_capabilities
 
+sys.modules['glance_store.common.fs_mount'] = mock.Mock()
+from glance_store._drivers import cinder # noqa
+
 
 class FakeObject(object):
     def __init__(self, **kwargs):
@@ -56,11 +61,11 @@ class TestCinderStore(base.StoreBaseTest,
         fake_sc = [{u'endpoints': [{u'publicURL': u'http://foo/public_url'}],
                     u'endpoints_links': [],
                     u'name': u'cinder',
-                    u'type': u'volumev2'}]
+                    u'type': u'volumev3'}]
         self.context = FakeObject(service_catalog=fake_sc,
-                                  user='fake_user',
+                                  user_id='fake_user',
                                   auth_token='fake_token',
-                                  tenant='fake_tenant')
+                                  project_id='fake_project')
         self.hash_algo = 'sha256'
 
     def test_get_cinderclient(self):
@@ -147,7 +152,8 @@ class TestCinderStore(base.StoreBaseTest,
 
     def _test_open_cinder_volume(self, open_mode, attach_mode, error,
                                  multipath_supported=False,
-                                 enforce_multipath=False):
+                                 enforce_multipath=False,
+                                 encrypted_nfs=False):
         self.config(cinder_mount_point_base=None)
         fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
         fake_volumes = FakeObject(get=lambda id: fake_volume,
@@ -189,19 +195,35 @@ class TestCinderStore(base.StoreBaseTest,
                                    'get_connector_properties') as mock_conn:
                 if error:
                     self.assertRaises(error, do_open)
+                elif encrypted_nfs:
+                    fake_volume.initialize_connection.return_value = {
+                        'driver_volume_type': 'nfs'
+                    }
+                    fake_volume.encrypted = True
+                    try:
+                        with self.store._open_cinder_volume(
+                                fake_client, fake_volume, open_mode):
+                            pass
+                    except exceptions.BackendException:
+                        self.assertEqual(1,
+                                         fake_volume.unreserve.call_count)
+                        self.assertEqual(1,
+                                         fake_volume.delete.call_count)
                 else:
                     do_open()
 
-                mock_conn.assert_called_once_with(
-                    root_helper, socket.gethostname(), multipath_supported,
-                    enforce_multipath)
-                fake_connector.connect_volume.assert_called_once_with(mock.ANY)
-                fake_connector.disconnect_volume.assert_called_once_with(
-                    mock.ANY, fake_devinfo)
-                fake_volume.attach.assert_called_once_with(
-                    None, 'glance_store', attach_mode,
-                    host_name=socket.gethostname())
-                fake_volumes.detach.assert_called_once_with(fake_volume)
+                if not encrypted_nfs:
+                    mock_conn.assert_called_once_with(
+                        root_helper, socket.gethostname(),
+                        multipath_supported, enforce_multipath)
+                    fake_connector.connect_volume.assert_called_once_with(
+                        mock.ANY)
+                    fake_connector.disconnect_volume.assert_called_once_with(
+                        mock.ANY, fake_devinfo)
+                    fake_volume.attach.assert_called_once_with(
+                        None, 'glance_store', attach_mode,
+                        host_name=socket.gethostname())
+                    fake_volumes.detach.assert_called_once_with(fake_volume)
 
     def test_open_cinder_volume_rw(self):
         self._test_open_cinder_volume('wb', 'rw', None)
@@ -224,6 +246,9 @@ class TestCinderStore(base.StoreBaseTest,
                                       multipath_supported=True,
                                       enforce_multipath=True)
 
+    def test_open_cinder_volume_nfs_encrypted(self):
+        self._test_open_cinder_volume('rb', 'ro', None, encrypted_nfs=True)
+
     def test_cinder_configure_add(self):
         self.assertRaises(exceptions.BadStoreConfiguration,
                           self.store._check_context, None)
@@ -309,7 +334,8 @@ class TestCinderStore(base.StoreBaseTest,
         expected_size = size_kb * units.Ki
         expected_file_contents = b"*" * expected_size
         image_file = six.BytesIO(expected_file_contents)
-        expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_checksum = md5(expected_file_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = 'cinder://%s' % fake_volume.id
         fake_client = FakeObject(auth_token=None, management_url=None)
@@ -337,7 +363,7 @@ class TestCinderStore(base.StoreBaseTest,
             fake_volumes.create.assert_called_once_with(
                 1,
                 name='image-%s' % expected_image_id,
-                metadata={'image_owner': self.context.tenant,
+                metadata={'image_owner': self.context.project_id,
                           'glance_image_id': expected_image_id,
                           'image_size': str(expected_size)},
                 volume_type='some_type')
@@ -384,3 +410,26 @@ class TestCinderStore(base.StoreBaseTest,
             loc = location.get_location_from_uri(uri, conf=self.conf)
             self.store.delete(loc, context=self.context)
             fake_volume.delete.assert_called_once_with()
+
+    def test_set_url_prefix(self):
+        self.assertEqual('cinder://', self.store._url_prefix)
+
+    def test_configure_add(self):
+
+        def fake_volume_type(name):
+            if name != 'some_type':
+                raise cinder.cinder_exception.NotFound(code=404)
+
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=fake_volume_type))
+            self.config(cinder_volume_type='some_type')
+            # If volume type exists, no exception is raised
+            self.store.configure_add()
+            # setting cinder_volume_type to non-existent value will log a
+            # warning
+            self.config(cinder_volume_type='some_random_type')
+            with mock.patch.object(cinder, 'LOG') as mock_log:
+                self.store.configure_add()
+                mock_log.warning.assert_called_with(
+                    "Invalid `cinder_volume_type some_random_type`")
