@@ -15,11 +15,12 @@
 
 import contextlib
 import errno
-import hashlib
-import mock
 import os
+from unittest import mock
+
 import six
 import socket
+import sys
 import tempfile
 import time
 import uuid
@@ -28,15 +29,18 @@ import fixtures
 from os_brick.initiator import connector
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 
 import glance_store as store
-from glance_store._drivers import cinder
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
 from glance_store.tests.unit import test_store_capabilities as test_cap
 
+sys.modules['glance_store.common.fs_mount'] = mock.Mock()
+from glance_store._drivers import cinder # noqa
+
 
 class FakeObject(object):
     def __init__(self, **kwargs):
@@ -83,14 +87,20 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         fake_sc = [{u'endpoints': [{u'publicURL': u'http://foo/public_url'}],
                     u'endpoints_links': [],
                     u'name': u'cinder',
-                    u'type': u'volumev2'}]
+                    u'type': u'volumev3'}]
         self.context = FakeObject(service_catalog=fake_sc,
-                                  user='fake_user',
+                                  user_id='fake_user',
                                   auth_token='fake_token',
-                                  tenant='fake_tenant')
+                                  project_id='fake_project')
+        self.fake_admin_context = mock.MagicMock()
+        self.fake_admin_context.elevated.return_value = FakeObject(
+            service_catalog=fake_sc,
+            user_id='admin_user',
+            auth_token='admin_token',
+            project_id='admin_project')
 
     def test_location_url_prefix_is_set(self):
-        self.assertEqual("cinder://", self.store.url_prefix)
+        self.assertEqual("cinder://cinder1", self.store.url_prefix)
 
     def test_get_cinderclient(self):
         cc = self.store.get_cinderclient(self.context)
@@ -106,6 +116,14 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         self.assertIsNone(cc.client.auth_token)
         self.assertEqual('test_address', cc.client.management_url)
 
+    def test_get_cinderclient_legacy_update(self):
+        cc = self.store.get_cinderclient(self.fake_admin_context,
+                                         legacy_update=True)
+        self.assertEqual('admin_token', cc.client.auth_token)
+        self.assertEqual('admin_user', cc.client.user)
+        self.assertEqual('admin_project', cc.client.projectid)
+        self.assertEqual('http://foo/public_url', cc.client.management_url)
+
     def test_temporary_chown(self):
         class fake_stat(object):
             st_uid = 1
@@ -243,7 +261,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                                       multipath_supported=True,
                                       enforce_multipath=True)
 
-    def test_cinder_configure_add(self):
+    def test_cinder_check_context(self):
         self.assertRaises(exceptions.BadStoreConfiguration,
                           self.store._check_context, None)
 
@@ -253,6 +271,85 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
 
         self.store._check_context(FakeObject(service_catalog='fake'))
 
+    def test_configure_add(self):
+
+        def fake_volume_type_check(name):
+            if name != 'some_type':
+                raise cinder.cinder_exception.NotFound(code=404)
+
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=fake_volume_type_check))
+            self.config(cinder_volume_type='some_type',
+                        group=self.store.backend_group)
+            # If volume type exists, no exception is raised
+            self.store.configure_add()
+            # setting cinder_volume_type to non-existent value will log a
+            # warning
+            self.config(cinder_volume_type='some_random_type',
+                        group=self.store.backend_group)
+            with mock.patch.object(cinder, 'LOG') as mock_log:
+                self.store.configure_add()
+                mock_log.warning.assert_called_with(
+                    "Invalid `cinder_volume_type some_random_type`")
+
+    def test_configure_add_cinder_service_down(self):
+
+        def fake_volume_type_check(name):
+            raise cinder.cinder_exception.ClientException(code=503)
+
+        self.config(cinder_volume_type='some_type',
+                    group=self.store.backend_group)
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=fake_volume_type_check))
+            # We handle the ClientException to pass so no exception is raised
+            # in this case
+            self.store.configure_add()
+
+    def test_configure_add_authorization_failed(self):
+
+        def fake_volume_type_check(name):
+            raise cinder.exceptions.AuthorizationFailure(code=401)
+
+        self.config(cinder_volume_type='some_type',
+                    group=self.store.backend_group)
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=fake_volume_type_check))
+            # Anything apart from invalid volume type or cinder service
+            # down will raise an exception
+            self.assertRaises(cinder.exceptions.AuthorizationFailure,
+                              self.store.configure_add)
+
+    def test_is_image_associated_with_store(self):
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volumes=FakeObject(
+                get=lambda volume_id: FakeObject(volume_type='some_type')),
+                volume_types=FakeObject(
+                    default=lambda: FakeObject(name='some_type')))
+            # When cinder_volume_type is set and is same as volume's type
+            self.config(cinder_volume_type='some_type',
+                        group=self.store.backend_group)
+            fake_vol_id = str(uuid.uuid4())
+            type_match = self.store.is_image_associated_with_store(
+                self.context, fake_vol_id)
+            self.assertTrue(type_match)
+            # When cinder_volume_type is not set and volume's type is same as
+            # set default volume type
+            self.config(cinder_volume_type=None,
+                        group=self.store.backend_group)
+            type_match = self.store.is_image_associated_with_store(
+                self.context, fake_vol_id)
+            self.assertTrue(type_match)
+            # When cinder_volume_type is not set and volume's type does not
+            # match with default volume type
+            mocked_cc.return_value.volume_types = FakeObject(
+                default=lambda: {'name': 'random_type'})
+            type_match = self.store.is_image_associated_with_store(
+                self.context, fake_vol_id)
+            self.assertFalse(type_match)
+
     def test_cinder_get(self):
         expected_size = 5 * units.Ki
         expected_file_contents = b"*" * expected_size
@@ -275,7 +372,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
                                               volumes=fake_volumes)
-            uri = "cinder://%s" % fake_volume_uuid
+            uri = "cinder://cinder1/%s" % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
@@ -302,7 +399,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
-            uri = 'cinder://%s' % fake_volume_uuid
+            uri = 'cinder://cinder1/%s' % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
@@ -321,7 +418,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
-            uri = 'cinder://%s' % fake_volume_uuid
+            uri = 'cinder://cinder1/%s' % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
@@ -334,8 +431,9 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         expected_size = size_kb * units.Ki
         expected_file_contents = b"*" * expected_size
         image_file = six.BytesIO(expected_file_contents)
-        expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
-        expected_location = 'cinder://%s' % fake_volume.id
+        expected_checksum = md5(expected_file_contents,
+                                usedforsecurity=False).hexdigest()
+        expected_location = 'cinder://%s/%s' % (backend, fake_volume.id)
         fake_client = FakeObject(auth_token=None, management_url=None)
         fake_volume.manager.get.return_value = fake_volume
         fake_volumes = FakeObject(create=mock.Mock(return_value=fake_volume))
@@ -362,7 +460,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             fake_volumes.create.assert_called_once_with(
                 1,
                 name='image-%s' % expected_image_id,
-                metadata={'image_owner': self.context.tenant,
+                metadata={'image_owner': self.context.project_id,
                           'glance_image_id': expected_image_id,
                           'image_size': str(expected_size)},
                 volume_type='some_type')
@@ -406,7 +504,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
-            uri = 'cinder://%s' % fake_volume_uuid
+            uri = 'cinder://cinder1/%s' % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
