@@ -16,14 +16,15 @@
 """Tests the Multiple S3 backend store"""
 
 import hashlib
+from unittest import mock
 import uuid
 
 import boto3
 import botocore
 from botocore import exceptions as boto_exceptions
 from botocore import stub
-import mock
 from oslo_config import cfg
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 import six
 
@@ -205,7 +206,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_s3_size = FIVE_KB
         expected_s3_contents = b"*" * expected_s3_size
-        expected_checksum = hashlib.md5(expected_s3_contents).hexdigest()
+        expected_checksum = md5(expected_s3_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_s3_contents).hexdigest()
         expected_location = format_s3_location(
             S3_CONF['s3_store_access_key'],
@@ -258,7 +260,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_s3_size = FIVE_KB
         expected_s3_contents = b"*" * expected_s3_size
-        expected_checksum = hashlib.md5(expected_s3_contents).hexdigest()
+        expected_checksum = md5(expected_s3_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_s3_contents).hexdigest()
         expected_location = format_s3_location(
             S3_CONF['s3_store_access_key'],
@@ -331,7 +334,8 @@ class TestMultiS3Store(base.MultiStoreBaseTest,
         expected_image_id = str(uuid.uuid4())
         expected_s3_size = 16 * units.Mi
         expected_s3_contents = b"*" * expected_s3_size
-        expected_checksum = hashlib.md5(expected_s3_contents).hexdigest()
+        expected_checksum = md5(expected_s3_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(expected_s3_contents).hexdigest()
         expected_location = format_s3_location(
             S3_CONF['s3_store_access_key'],
