@@ -14,7 +14,9 @@
 #    under the License.
 
 import hashlib
-import mock
+from unittest import mock
+
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 import six
 
@@ -23,6 +25,7 @@ from glance_store import exceptions
 from glance_store import location as g_location
 from glance_store.tests import base
 from glance_store.tests.unit import test_store_capabilities
+from glance_store.tests import utils as test_utils
 
 
 class TestException(Exception):
@@ -34,6 +37,9 @@ class MockRados(object):
     class Error(Exception):
         pass
 
+    class ObjectNotFound(Exception):
+        pass
+
     class ioctx(object):
         def __init__(self, *args, **kwargs):
             pass
@@ -121,7 +127,7 @@ class MockRBD(object):
             raise NotImplementedError()
 
         def resize(self, *args, **kwargs):
-            raise NotImplementedError()
+            pass
 
         def discard(self, offset, length):
             raise NotImplementedError()
@@ -164,6 +170,123 @@ class MockRBD(object):
     RBD_FEATURE_LAYERING = 1
 
 
+class TestReSize(base.StoreBaseTest,
+                 test_store_capabilities.TestStoreCapabilitiesChecking):
+
+    def setUp(self):
+        """Establish a clean test environment."""
+        super(TestReSize, self).setUp()
+
+        rbd_store.rados = MockRados
+        rbd_store.rbd = MockRBD
+
+        self.store = rbd_store.Store(self.conf)
+        self.store.configure()
+        self.store_specs = {'pool': 'fake_pool',
+                            'image': 'fake_image',
+                            'snapshot': 'fake_snapshot'}
+        self.location = rbd_store.StoreLocation(self.store_specs,
+                                                self.conf)
+        self.hash_algo = 'sha256'
+
+    def test_add_w_image_size_zero_less_resizes(self):
+        """Assert that correct size is returned even though 0 was provided."""
+        data_len = 57 * units.Mi
+        data_iter = test_utils.FakeData(data_len)
+        with mock.patch.object(rbd_store.rbd.Image, 'resize') as resize:
+            with mock.patch.object(rbd_store.rbd.Image, 'write') as write:
+                ret = self.store.add(
+                    'fake_image_id', data_iter, 0, self.hash_algo)
+
+                # We expect to trim at the end so +1
+                expected = 1
+                expected_calls = []
+                data_len_temp = data_len
+                resize_amount = self.store.WRITE_CHUNKSIZE
+                while data_len_temp > 0:
+                    expected_calls.append(resize_amount + (data_len -
+                                                           data_len_temp))
+                    data_len_temp -= resize_amount
+                    resize_amount *= 2
+                    expected += 1
+                self.assertEqual(expected, resize.call_count)
+                resize.assert_has_calls([mock.call(call) for call in
+                                         expected_calls])
+                expected = ([self.store.WRITE_CHUNKSIZE for i in range(int(
+                            data_len / self.store.WRITE_CHUNKSIZE))] +
+                            [(data_len % self.store.WRITE_CHUNKSIZE)])
+                actual = ([len(args[0]) for args, kwargs in
+                          write.call_args_list])
+                self.assertEqual(expected, actual)
+                self.assertEqual(data_len,
+                                 resize.call_args_list[-1][0][0])
+                self.assertEqual(data_len, ret[1])
+
+    def test_resize_on_write_ceiling(self):
+        image = mock.MagicMock()
+
+        # image, size, written, chunk
+
+        # Non-zero image size means no resize
+        ret = self.store._resize_on_write(image, 32, 16, 16)
+        self.assertEqual(0, ret)
+        image.resize.assert_not_called()
+
+        # Current size is smaller than we need
+        self.store.size = 8
+        ret = self.store._resize_on_write(image, 0, 16, 16)
+        self.assertEqual(8 + self.store.WRITE_CHUNKSIZE, ret)
+        self.assertEqual(self.store.WRITE_CHUNKSIZE * 2,
+                         self.store.resize_amount)
+        image.resize.assert_called_once_with(ret)
+
+        # More reads under the limit do not require a resize
+        image.resize.reset_mock()
+        self.store.size = ret
+        ret = self.store._resize_on_write(image, 0, 64, 16)
+        self.assertEqual(8 + self.store.WRITE_CHUNKSIZE, ret)
+        image.resize.assert_not_called()
+
+        # Read past the limit triggers another resize
+        ret = self.store._resize_on_write(image, 0, ret + 1, 16)
+        self.assertEqual(8 + self.store.WRITE_CHUNKSIZE * 3, ret)
+        image.resize.assert_called_once_with(ret)
+        self.assertEqual(self.store.WRITE_CHUNKSIZE * 4,
+                         self.store.resize_amount)
+
+        # Check that we do not resize past the 8G ceiling.
+
+        # Start with resize_amount at 4G, 1G read so far
+        image.resize.reset_mock()
+        self.store.resize_amount = 4 * units.Gi
+        self.store.size = 1 * units.Gi
+
+        # First resize happens and we get the 4G,
+        # resize_amount goes to limit of 8G
+        ret = self.store._resize_on_write(image, 0, 4097 * units.Mi, 16)
+        self.assertEqual(5 * units.Gi, ret)
+        self.assertEqual(8 * units.Gi, self.store.resize_amount)
+        self.store.size = ret
+
+        # Second resize happens and we get to 13G,
+        # resize amount stays at limit of 8G
+        ret = self.store._resize_on_write(image, 0, 6144 * units.Mi, 16)
+        self.assertEqual((5 + 8) * units.Gi, ret)
+        self.assertEqual(8 * units.Gi, self.store.resize_amount)
+        self.store.size = ret
+
+        # Third resize happens and we get to 21G,
+        # resize amount stays at limit of 8G
+        ret = self.store._resize_on_write(image, 0, 14336 * units.Mi, 16)
+        self.assertEqual((5 + 8 + 8) * units.Gi, ret)
+        self.assertEqual(8 * units.Gi, self.store.resize_amount)
+
+        image.resize.assert_has_calls([
+            mock.call(5 * units.Gi),
+            mock.call(13 * units.Gi),
+            mock.call(21 * units.Gi)])
+
+
 class TestStore(base.StoreBaseTest,
                 test_store_capabilities.TestStoreCapabilitiesChecking):
 
@@ -189,6 +312,9 @@ class TestStore(base.StoreBaseTest,
         self.data_iter = six.BytesIO(b'*' * self.data_len)
         self.hash_algo = 'sha256'
 
+    def test_thin_provisioning_is_disabled_by_default(self):
+        self.assertEqual(self.store.thin_provisioning, False)
+
     def test_add_w_image_size_zero(self):
         """Assert that correct size is returned even though 0 was provided."""
         self.store.chunk_size = units.Ki
@@ -291,7 +417,8 @@ class TestStore(base.StoreBaseTest,
         file_size = 5 * units.Ki  # 5K
         file_contents = b"*" * file_size
         image_file = six.BytesIO(file_contents)
-        expected_checksum = hashlib.md5(file_contents).hexdigest()
+        expected_checksum = md5(file_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(file_contents).hexdigest()
 
         with mock.patch.object(rbd_store.rbd.Image, 'write'):
@@ -301,6 +428,81 @@ class TestStore(base.StoreBaseTest,
         self.assertEqual(expected_checksum, checksum)
         self.assertEqual(expected_multihash, multihash)
 
+    def test_add_thick_provisioning_with_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes chunks is fully
+        written to rbd backend in a thick provisioning configuration.
+        """
+        chunk_size = units.Mi
+        content = b"*" * chunk_size + b"\x00" * chunk_size + b"*" * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 3, False)
+
+    def test_add_thin_provisioning_with_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes chunks is sparsified
+        in rbd backend with a thin provisioning configuration.
+        """
+        chunk_size = units.Mi
+        content = b"*" * chunk_size + b"\x00" * chunk_size + b"*" * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 2, True)
+
+    def test_add_thick_provisioning_without_holes_in_file(self):
+        """
+        Tests that a file which not contain null bytes chunks is fully
+        written to rbd backend in a thick provisioning configuration.
+        """
+        chunk_size = units.Mi
+        content = b"*" * 3 * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 3, False)
+
+    def test_add_thin_provisioning_without_holes_in_file(self):
+        """
+        Tests that a file which not contain null bytes chunks is fully
+        written to rbd backend in a thin provisioning configuration.
+        """
+        chunk_size = units.Mi
+        content = b"*" * 3 * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 3, True)
+
+    def test_add_thick_provisioning_with_partial_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes not aligned with
+        chunk size is fully written with a thick provisioning configuration.
+        """
+        chunk_size = units.Mi
+        my_chunk = int(chunk_size * 1.5)
+        content = b"*" * my_chunk + b"\x00" * my_chunk + b"*" * my_chunk
+        self._do_test_thin_provisioning(content, 3 * my_chunk, 5, False)
+
+    def test_add_thin_provisioning_with_partial_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes not aligned with
+        chunk size is sparsified with a thin provisioning configuration.
+        """
+        chunk_size = units.Mi
+        my_chunk = int(chunk_size * 1.5)
+        content = b"*" * my_chunk + b"\x00" * my_chunk + b"*" * my_chunk
+        self._do_test_thin_provisioning(content, 3 * my_chunk, 4, True)
+
+    def _do_test_thin_provisioning(self, content, size, write, thin):
+        self.config(rbd_store_chunk_size=1,
+                    rbd_thin_provisioning=thin)
+        self.store.configure()
+
+        image_id = 'fake_image_id'
+        image_file = six.BytesIO(content)
+        expected_checksum = md5(content,
+                                usedforsecurity=False).hexdigest()
+        expected_multihash = hashlib.sha256(content).hexdigest()
+
+        with mock.patch.object(rbd_store.rbd.Image, 'write') as mock_write:
+            loc, size, checksum, multihash, _ = self.store.add(
+                image_id, image_file, size, self.hash_algo)
+            self.assertEqual(mock_write.call_count, write)
+
+        self.assertEqual(expected_checksum, checksum)
+        self.assertEqual(expected_multihash, multihash)
+
     def test_delete(self):
         def _fake_remove(*args, **kwargs):
             self.called_commands_actual.append('remove')
@@ -450,6 +652,7 @@ class TestStore(base.StoreBaseTest,
     @mock.patch.object(MockRados.Rados, 'connect', side_effect=MockRados.Error)
     def test_rados_connect_error(self, _):
         rbd_store.rados.Error = MockRados.Error
+        rbd_store.rados.ObjectNotFound = MockRados.ObjectNotFound
 
         def test():
             with self.store.get_connection('conffile', 'rados_id'):
