@@ -16,14 +16,16 @@
 """Tests the Swift backend store"""
 
 import copy
+from unittest import mock
+
 import fixtures
 import hashlib
-import mock
 import tempfile
 import uuid
 
 from oslo_config import cfg
 from oslo_utils import encodeutils
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 import requests_mock
 import six
@@ -47,9 +49,9 @@ from glance_store.tests.unit import test_store_capabilities
 
 CONF = cfg.CONF
 
-FAKE_UUID = lambda: str(uuid.uuid4())
-FAKE_UUID2 = lambda: str(uuid.uuid4())
-FAKE_UUID3 = lambda: str(uuid.uuid4())
+FAKE_UUID = lambda: str(uuid.uuid4())  # noqa: E731
+FAKE_UUID2 = lambda: str(uuid.uuid4())  # noqa: E731
+FAKE_UUID3 = lambda: str(uuid.uuid4())  # noqa: E731
 
 
 Store = swift.Store
@@ -120,7 +122,8 @@ class SwiftTests(object):
                 if kwargs.get('headers'):
                     manifest = kwargs.get('headers').get('X-Object-Manifest')
                     etag = kwargs.get('headers') \
-                                 .get('ETag', hashlib.md5(b'').hexdigest())
+                                 .get('ETag', md5(
+                                     b'', usedforsecurity=False).hexdigest())
                     fixture_headers[fixture_key] = {
                         'manifest': True,
                         'etag': etag,
@@ -132,7 +135,7 @@ class SwiftTests(object):
                     fixture_object = six.BytesIO()
                     read_len = 0
                     chunk = contents.read(CHUNKSIZE)
-                    checksum = hashlib.md5()
+                    checksum = md5(usedforsecurity=False)
                     while chunk:
                         fixture_object.write(chunk)
                         read_len += len(chunk)
@@ -142,7 +145,8 @@ class SwiftTests(object):
                 else:
                     fixture_object = six.BytesIO(contents)
                     read_len = len(contents)
-                    etag = hashlib.md5(fixture_object.getvalue()).hexdigest()
+                    etag = md5(fixture_object.getvalue(),
+                               usedforsecurity=False).hexdigest()
                 if read_len > MAX_SWIFT_OBJECT_SIZE:
                     msg = ('Image size:%d exceeds Swift max:%d' %
                            (read_len, MAX_SWIFT_OBJECT_SIZE))
@@ -420,7 +424,8 @@ class SwiftTests(object):
         self.store.configure()
         expected_swift_size = FIVE_KB
         expected_swift_contents = b"*" * expected_swift_size
-        expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
+        expected_checksum = md5(expected_swift_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(
             expected_swift_contents).hexdigest()
         expected_image_id = str(uuid.uuid4())
@@ -541,7 +546,7 @@ class SwiftTests(object):
             expected_swift_size = FIVE_KB
             expected_swift_contents = b"*" * expected_swift_size
             expected_checksum = \
-                hashlib.md5(expected_swift_contents).hexdigest()
+                md5(expected_swift_contents, usedforsecurity=False).hexdigest()
             expected_multihash = \
                 hashlib.sha256(expected_swift_contents).hexdigest()
 
@@ -617,7 +622,8 @@ class SwiftTests(object):
         """
         expected_swift_size = FIVE_KB
         expected_swift_contents = b"*" * expected_swift_size
-        expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
+        expected_checksum = md5(expected_swift_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = \
             hashlib.sha256(expected_swift_contents).hexdigest()
         expected_image_id = str(uuid.uuid4())
@@ -663,7 +669,8 @@ class SwiftTests(object):
         """
         expected_swift_size = FIVE_KB
         expected_swift_contents = b"*" * expected_swift_size
-        expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
+        expected_checksum = md5(expected_swift_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = \
             hashlib.sha256(expected_swift_contents).hexdigest()
         expected_image_id = str(uuid.uuid4())
@@ -884,7 +891,8 @@ class SwiftTests(object):
         """
         expected_swift_size = FIVE_KB
         expected_swift_contents = b"*" * expected_swift_size
-        expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
+        expected_checksum = md5(expected_swift_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = \
             hashlib.sha256(expected_swift_contents).hexdigest()
         expected_image_id = str(uuid.uuid4())
@@ -938,7 +946,8 @@ class SwiftTests(object):
         # Set up a 'large' image of 5KB
         expected_swift_size = FIVE_KB
         expected_swift_contents = b"*" * expected_swift_size
-        expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
+        expected_checksum = md5(expected_swift_contents,
+                                usedforsecurity=False).hexdigest()
         expected_multihash = \
             hashlib.sha256(expected_swift_contents).hexdigest()
         expected_image_id = str(uuid.uuid4())
@@ -1311,7 +1320,7 @@ class SwiftTests(object):
         mock_identity.V3Token.assert_called_once_with(
             auth_url=default_swift_reference.get('auth_address'),
             token=ctxt.auth_token,
-            project_id=ctxt.tenant
+            project_id=ctxt.project_id
         )
         mock_session.Session.assert_any_call(auth=mock_identity.V3Token(),
                                              verify=verify)
@@ -1333,8 +1342,8 @@ class SwiftTests(object):
                                              verify=verify)
         mock_client.Client.assert_any_call(session=trustee_session)
         trustor_client.trusts.create.assert_called_once_with(
-            trustee_user='fake_user', trustor_user=ctxt.user,
-            project=ctxt.tenant, impersonation=True,
+            trustee_user='fake_user', trustor_user=ctxt.user_id,
+            project=ctxt.project_id, impersonation=True,
             role_names=['fake_role']
         )
         mock_identity.V3Password.assert_any_call(
@@ -1921,14 +1930,14 @@ class TestChunkReader(base.StoreBaseTest):
         """
         CHUNKSIZE = 100
         data = b'*' * units.Ki
-        expected_checksum = hashlib.md5(data).hexdigest()
+        expected_checksum = md5(data, usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(data).hexdigest()
         data_file = tempfile.NamedTemporaryFile()
         data_file.write(data)
         data_file.flush()
         infile = open(data_file.name, 'rb')
         bytes_read = 0
-        checksum = hashlib.md5()
+        checksum = md5(usedforsecurity=False)
         os_hash_value = hashlib.sha256()
         while True:
             cr = swift.ChunkReader(infile, checksum, os_hash_value, CHUNKSIZE)
@@ -1950,10 +1959,10 @@ class TestChunkReader(base.StoreBaseTest):
         Replicate what goes on in the Swift driver with the
         repeated creation of the ChunkReader object
         """
-        expected_checksum = hashlib.md5(b'').hexdigest()
+        expected_checksum = md5(b'', usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(b'').hexdigest()
         CHUNKSIZE = 100
-        checksum = hashlib.md5()
+        checksum = md5(usedforsecurity=False)
         os_hash_value = hashlib.sha256()
         data_file = tempfile.NamedTemporaryFile()
         infile = open(data_file.name, 'rb')
@@ -2049,7 +2058,7 @@ class TestBufferedReader(base.StoreBaseTest):
         self.infile = six.BytesIO(s)
         self.infile.seek(0)
 
-        self.checksum = hashlib.md5()
+        self.checksum = md5(usedforsecurity=False)
         self.hash_algo = HASH_ALGO
         self.os_hash_value = hashlib.sha256()
         self.verifier = mock.MagicMock(name='mock_verifier')
@@ -2109,7 +2118,7 @@ class TestBufferedReader(base.StoreBaseTest):
 
     def test_checksums(self):
         # checksums are updated only once on a full segment read
-        expected_csum = hashlib.md5()
+        expected_csum = md5(usedforsecurity=False)
         expected_csum.update(b'1234567')
         expected_multihash = hashlib.sha256()
         expected_multihash.update(b'1234567')
@@ -2121,7 +2130,7 @@ class TestBufferedReader(base.StoreBaseTest):
     def test_checksum_updated_only_once_w_full_segment_read(self):
         # Test that checksums are updated only once when a full segment read
         # is followed by a seek and partial reads.
-        expected_csum = hashlib.md5()
+        expected_csum = md5(usedforsecurity=False)
         expected_csum.update(b'1234567')
         expected_multihash = hashlib.sha256()
         expected_multihash.update(b'1234567')
@@ -2136,7 +2145,7 @@ class TestBufferedReader(base.StoreBaseTest):
     def test_checksum_updates_during_partial_segment_reads(self):
         # Test to check that checksums are updated with only the bytes
         # not seen when the number of bytes being read is changed
-        expected_csum = hashlib.md5()
+        expected_csum = md5(usedforsecurity=False)
         expected_multihash = hashlib.sha256()
         self.reader.read(4)
         expected_csum.update(b'1234')
@@ -2159,7 +2168,7 @@ class TestBufferedReader(base.StoreBaseTest):
 
     def test_checksum_rolling_calls(self):
         # Test that the checksum continues on to the next segment
-        expected_csum = hashlib.md5()
+        expected_csum = md5(usedforsecurity=False)
         expected_multihash = hashlib.sha256()
         self.reader.read(7)
         expected_csum.update(b'1234567')
@@ -2228,7 +2237,7 @@ class TestBufferedReader(base.StoreBaseTest):
         infile = six.BytesIO(s)
         infile.seek(0)
         total = 7
-        checksum = hashlib.md5()
+        checksum = md5(usedforsecurity=False)
         os_hash_value = hashlib.sha256()
         self.reader = buffered.BufferedReader(
             infile, checksum, os_hash_value, total)
@@ -2253,14 +2262,14 @@ class TestBufferedReader(base.StoreBaseTest):
         """
         CHUNKSIZE = 100
         data = b'*' * units.Ki
-        expected_checksum = hashlib.md5(data).hexdigest()
+        expected_checksum = md5(data, usedforsecurity=False).hexdigest()
         expected_multihash = hashlib.sha256(data).hexdigest()
         data_file = tempfile.NamedTemporaryFile()
         data_file.write(data)
         data_file.flush()
         infile = open(data_file.name, 'rb')
         bytes_read = 0
-        checksum = hashlib.md5()
+        checksum = md5(usedforsecurity=False)
         os_hash_value = hashlib.sha256()
         while True:
             cr = buffered.BufferedReader(infile, checksum, os_hash_value,
